####################################
###   BOMBER RUN v1.07 for tml   ###
### by Piotr Kowalewski (komame) ###
### This is a demonstration game ###
###     using the tml module     ###
###         for HP Prime         ###
###     Sep 26 - Oct 8, 2024     ###
####################################

import tml
import sys
import urandom
from hpprime import eval as ppleval

# Grid dimensions
ROWS = const(28)
COLUMNS = const(40)

# Building factors
TOTAL_XS = const(300)
MAX_XS = const(25)

# Initialize terminal (tml)
t=tml.tml(font='atari8x8',dark_mode=True)

### Function declarations ###
def wait(ms):
  ppleval('WAIT(%d/1e3)' % ms)

# Crash animation
def crash(x,y):
  s = '@O&#%*!$o@ '
  for c in s:
    t.print_xy(x,y,c)
    wait(50)

def wait_keys_released():
  while t.get_keys() != []:
    wait(1)

def refresh_score(score, misses):
  t.set_status('Destruction: %.1f%%, Misses: %d' % (score * 100 / TOTAL_XS, misses))
 
# Title screen printing using ASCII-art
def show_title():
  t.clear()
  t.print(" ____                  _\n| __ )  ___  _ __ ___ | |__   ___ _ __\n|  _ \\ / _ \\| '_ ` _ \\| '_ \\ / _ \\ '__|\n| |_) | (_) | | | | | | |_) |  __/ |\n|____/ \\___/|_| |_| |_|_.__/ \\___|_|")
  t.print(" ____\n|  _ \\ _   _ _ __\n| |_) | | | | '_ \\\n|  _ <| |_| | | | |\n|_| \\_\\\\__,_|_| |_| v1.07 for tml\n")
  t.print('by Piotr Kowalewski, September 2024\n%s' % ('_'*40))
  t.print('Instructions:\n\n\tAs a bomber pilot, your task is to\ndestroy all the buildings in the enemy\nbase and land on hostile territory to\ncapture it.\n\n\tTo drop a bomb, press Enter.\nThe higher the altitude from which the  bomb strikes the building, the greater  the destruction it will cause. You can  drop up to three bombs during a single  run.')
  t.set_status('Esc: exit, Other key: start game')
  wait_keys_released()
 
  # Wait until key is pressed
  while 1:
    k = t.get_keys()
    if 4 in k:  # Esc
        sys.exit(0)
    elif k != []:  # Any other key
        break;
    wait(2)

  # Wait until key is released and clear screen
  wait_keys_released()
  t.clear()

def bomber_run():
  while True:
  # NEW GAME INITIALIZATION #

  # Show title screen
    show_title()

  # Generate 32 random numbers that sum up to 200
    middle_columns = 32
    random_numbers = [0] * middle_columns
    remaining_Xs = TOTAL_XS

  # Distribute remaining Xs randomly
    while remaining_Xs > 0:
      index = urandom.randint(0, middle_columns - 1)
      if random_numbers[index] < MAX_XS:
        random_numbers[index] += 1
        remaining_Xs -= 1
    
  # Create an empty grid filled with spaces
    grid = [[' ' for _ in range(COLUMNS)] for _ in range(ROWS)]

  # Fill the middle columns with 'X' based on the randomly generated numbers
    for col in range(4, 36):  # Columns 4 to 35 (inclusive)
      num_Xs = random_numbers[col - 4]  # Adjust index to fill middle columns
      for row in range(ROWS - num_Xs, ROWS):
        grid[row][col] = 'X'

  # Display the grid
    s=''
    for row in grid:
      s+=''.join(row)
    t.print(s.rstrip(),end='')
    del s

  # INITAL VALUES
    px,py = 0,0  # Bomber previous coordinates
    bombs = []   # Falling bombs data
    bc = 0  # Bomb counter in a single run
    score = 0
    misses = 0
    y = 0
  
    lost = False

    key_hold = t.get_keys() != []
    refresh_score(score, misses)

  #################
  ### GAME LOOP ###
  #################

    while y < ROWS:
      for x in range(COLUMNS):
        if grid[y][x]!=' ':
          crash(px,py)
          lost = True
          break
        t.print_xy(px,py,' ')
        t.print_xy(x,y,'>')
        px,py = x,y
        if 30 in t.get_keys() and y < 28 and bc < 3 and not key_hold:
            bombs.append({'x':x,'y':y,'fact':5,'miss':1})
            bc += 1
            key_hold = True
        if t.get_keys() == []:
            key_hold = False
        if score == TOTAL_XS:
            wait((y << 1) + y)
            if y == 27 and x == 19: break
        wait(50);
        if score == TOTAL_XS and y < 27 and not x & 1:
          y += 1
        for b in bombs[:]:
          b['y'] += 1
          t.print_xy(b['x'],b['y']-1,' ')
          if b['y'] == 28:
            bombs.remove(b)
            if(grid[b['y']-1][b['x']] == ' '):
              misses += b['miss']
              refresh_score(score, misses)
            continue
          if grid[b['y']][b['x']] == 'X':
            t.print_xy(b['x'],b['y'],'&')
            b['fact'] -= 1
            b['miss'] = 0
            if b['fact'] == 0:
              bombs.remove(b)
            else:
              score += 1
              grid[b['y']][b['x']] = ' '
              refresh_score(score, misses)
          else:
            if b['y'] & 1: b['fact'] += 1
            t.print_xy(b['x'],b['y'],'v')
      if lost:
          break
      bc = 0
      y += 1

  # Game over
    if lost:
      t.print_xy(12,7,'+-------------+')
      t.print_xy(12,8,'|  GAME OVER  |')
      t.print_xy(12,9,'+-------------+')
    else:
      t.print_xy(10,7,'+------------------+')
      t.print_xy(10,8,'| CONGRATULATIONS! |')
      t.print_xy(10,9,'+------------------+')
    score = score - misses * 2
    if score < 0: score = 0
    t.set_status('Score: %d, Misses: %d' % (score,misses))

    t.read_key()
    
bomber_run()